import os
from dotenv import load_dotenv
load_dotenv()

# 把.env中的环境变量都加载进来
dashscope_api_key = os.getenv("DASHSCOPE_API_KEY")
bailian_app_id = os.getenv("BAILIAN_APP_ID")
bailian_app_id_report = os.getenv("BAILIAN_APP_ID_REPORT")
user_api_keyid = os.getenv("USER_API_KEYID")
secret_key = os.getenv("SECRET_KEY")
mcp_secret_key = os.getenv("MCP_SECRET_KEY")
api_base_url = os.getenv("API_BASE_URL")

mcp_name =os.getenv("MCP_NAME")
mcp_server_host = os.getenv("MCP_SERVER_HOST")
mcp_server_port = os.getenv("MCP_SERVER_PORT")
mcp_log_name = os.getenv("MCP_LOG_NAME")

report_mcp_name = os.getenv("REPORT_MCP_NAME")
report_mcp_server_host = os.getenv("REPORT_MCP_SERVER_HOST")
report_mcp_server_port = os.getenv("REPORT_MCP_SERVER_PORT")
report_mcp_log_name = os.getenv("REPORT_MCP_LOG_NAME")

upload_folder = os.getenv("UPLOAD_FOLDER")
alibaba_cloud_access_key_id = os.getenv("ALIBABA_CLOUD_ACCESS_KEY_ID")
alibaba_cloud_access_key_secret = os.getenv("ALIBABA_CLOUD_ACCESS_KEY_SECRET")
alibaba_cloud_categoryid = os.getenv("ALIBABA_CLOUD_CATEGORYID")
alibaba_cloud_workspaceid = os.getenv("ALIBABA_CLOUD_WORKSPACEID")
alibaba_cloud_categorytype = os.getenv("ALIBABA_CLOUD_CATEGORYTYPE")

web_server_host = os.getenv("WEB_SERVER_HOST")
web_server_port = os.getenv("WEB_SERVER_PORT")
web_log_name = os.getenv("WEB_LOG_NAME")

cytx_api_url = os.getenv("CYTX_API_URL")
cytx_api_getPlanData_url = cytx_api_url + "/api/aged/getPlanData"
cytx_api_getAgedInsurance_url = cytx_api_url + "/api/aged/getAgedInsurance"
cytx_api_getTargetList_url = cytx_api_url + "/api/aged/getTargetList"
cytx_api_getSolution_url = cytx_api_url + "/api/aged/getSolution"
cytx_api_saveBasicInfo_url = cytx_api_url + "/api/aged/saveBasicInfo"
cytx_api_saveAgeInsurance_url = cytx_api_url + "/api/aged/saveAgeInsurance"
cytx_api_saveTargetList_url = cytx_api_url + "/api/aged/saveTargetList"
cytx_api_saveSolution_url = cytx_api_url + "/api/aged/saveSolution"

cytx_api_select_report_url = cytx_api_url + "/api/pub/selectReportJson"
cytx_api_calc_target_list_url = cytx_api_url + "/api/whole/planCalc/calcTargetList"
cytx_api_get_plan_report_url = cytx_api_url + "/api/aged/getPlanReport"

type_code_list = [
  {
    "name": "个人信息",
    "code": "0-0-0"
  },
  {
    "name": "资产",
    "code": "1-0-0"
  },
  {
    "name": "资产-现金类资产",
    "code": "1-1-0"
  },
  {
    "name": "资产-现金类资产-活期存款",
    "code": "1-1-6"
  },
  {
    "name": "资产-现金类资产-货币基金",
    "code": "1-1-8"
  },
  {
    "name": "资产-固收类资产",
    "code": "1-2-0"
  },
  {
    "name": "资产-固收类资产-定期存款",
    "code": "1-2-7"
  },
  {
    "name": "资产-固收类资产-债券",
    "code": "1-2-11"
  },
  {
    "name": "资产-固收类资产-债券类基金",
    "code": "1-2-23"
  },
  {
    "name": "资产-固收类资产-理财产品",
    "code": "1-2-24"
  },
  {
    "name": "资产-权益类资产",
    "code": "1-3-0"
  },
  {
    "name": "资产-权益类资产-股票",
    "code": "1-3-9"
  },
  {
    "name": "资产-权益类资产-股票类基金",
    "code": "1-3-10"
  },
  {
    "name": "资产-权益类资产-私募基金",
    "code": "1-3-31"
  },
  {
    "name": "资产-房产类资产-房产",
    "code": "1-5-14"
  },
  {
    "name": "资产-个人养老金",
    "code": "1-46-46"
  },
  {
    "name": "资产-公积金账户",
    "code": "1-47-47"
  },
  {
    "name": "资产-社保养老账户",
    "code": "1-17-17"
  },
  {
    "name": "资产-企业年金",
    "code": "1-48-48"
  },
  {
    "name": "资产-其他资产-其他资产",
    "code": "1-4-12"
  },
  {
    "name": "资产-其他资产-贵金属",
    "code": "1-4-41"
  },
  {
    "name": "财务-收入-工作收入",
    "code": "2-1-1"
  },
  {
    "name": "财务-收入-经营性收入",
    "code": "2-1-2"
  },
  {
    "name": "财务-收入-其他长期收入(主动收入)",
    "code": "2-1-3"
  },
  {
    "name": "财务-收入-其他长期收入(被动收入)",
    "code": "2-1-4"
  },
  {
    "name": "财务-支出-生活支出",
    "code": "2-4-1"
  },
  {
    "name": "财务-支出-子女教育",
    "code": "2-4-2"
  },
  {
    "name": "财务-支出-赡养父母支出",
    "code": "2-4-3"
  },
  {
    "name": "财务-支出-旅游支出",
    "code": "2-4-4"
  },
  {
    "name": "财务-支出-娱乐支出",
    "code": "2-4-5"
  },
  {
    "name": "财务-支出-其他支出",
    "code": "2-4-6"
  },
  {
    "name": "养老目标",
    "code": "3-0-0"
  },
  {
    "name": "养老目标-医疗",
    "code": "3-1-0"
  },
  {
    "name": "养老目标-医疗-医保",
    "code": "3-1-1"
  },
  {
    "name": "养老目标-医疗-普通保障",
    "code": "3-1-2"
  },
  {
    "name": "养老目标-医疗-高端医疗保障",
    "code": "3-1-3"
  },
  {
    "name": "养老目标-医疗-慢病医疗",
    "code": "3-1-5"
  },
  {
    "name": "养老目标-医疗-重疾医疗",
    "code": "3-1-6"
  },
  {
    "name": "养老目标-居住",
    "code": "3-2-0"
  },
  {
    "name": "养老目标-居住-居住",
    "code": "3-2-1"
  },
  {
    "name": "养老目标-居住-养老院",
    "code": "3-2-2"
  },
  {
    "name": "养老目标-居住-养老社区",
    "code": "3-2-5"
  },
  {
    "name": "养老目标-护理",
    "code": "3-3-0"
  },
  {
    "name": "养老目标-护理-家人照顾",
    "code": "3-3-1"
  },
  {
    "name": "养老目标-护理-保姆照顾",
    "code": "3-3-2"
  },
  {
    "name": "养老目标-护理-护工照顾",
    "code": "3-3-1"
  },
  {
    "name": "养老目标-生活-无分类",
    "code": "3-6-0"
  },
  {
    "name": "养老目标-爱好-无分类",
    "code": "3-7-0"
  },
  {
    "name": "养老目标-自定义-无分类",
    "code": "3-9-0"
  },
  {
    "name": "保险-医疗险-无分类",
    "code": "4-1-0"
  },
  {
    "name": "保险-意外险-无分类",
    "code": "4-2-0"
  },
  {
    "name": "保险-重疾险-无分类",
    "code": "4-3-0"
  },
  {
    "name": "保险-养老年金险-无分类",
    "code": "4-4-0"
  },
  {
    "name": "保险-寿险-无分类",
    "code": "4-5-0"
  },
  {
    "name": "满意度",
    "code": "5-0-0"
  }
,
  {
    "name": "保险",
    "code": "4-0-0"
  }
]



from apscheduler.schedulers.blocking import BlockingScheduler
from datetime import datetime,timedelta
import os
from config.settings import upload_folder
import logging

logger = logging.getLogger(__name__)

def job():
    try:
        print(f"任务执行时间: {datetime.now()}")
        yesterday = (datetime.now() - timedelta(days=1)).strftime("%Y%m%d")
        target_dir = os.path.join(upload_folder, yesterday)

        logger.info(f"开始清理目录: {target_dir}")

        if not os.path.exists(target_dir):
            logger.warning(f"目录不存在: {target_dir}")
            return

        # 遍历并删除文件
        for root, dirs, files in os.walk(target_dir):
            for file in files:
                file_path = os.path.join(root, file)
                os.remove(file_path)
                logger.info(f"已删除文件: {file_path}")
        logger.info(f"清理完成: {target_dir}")
    except Exception as e:
        logger.error(f"清理任务出错: {e}", exc_info=True)




import json
import logging
import os
from logging.handlers import TimedRotatingFileHandler
from typing import Optional

from mcp.server.fastmcp import FastMCP

from config.settings import report_mcp_server_host, report_mcp_server_port, report_mcp_name, report_mcp_log_name, \
    mcp_secret_key
from models.report_model import ReportModel, json_to_model
from services.common_utils import safe_json_loads, safe_get
from services.requests_service import requests_select_report, requests_get_plan_report, requests_calc_target_list, \
    requests_get_target_list

# 初始化日志
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] %(name)s: %(message)s',
    handlers=[
        TimedRotatingFileHandler(report_mcp_log_name, when="D", backupCount=7),
        logging.StreamHandler()
    ]
)
logger = logging.getLogger(__name__)
logging.getLogger("mcp.server.sse").setLevel(logging.DEBUG)

dependencies = [
    "websockets"
]

mcp = FastMCP(report_mcp_name, port=int(report_mcp_server_port), host=report_mcp_server_host, dependencies=dependencies)


@mcp.tool(name="getInfo", description="get")
def get_info(sk: str, cytxToken: str, planId: str) -> str:
    logger.info("into get_info")
    try:
        logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s", sk, cytxToken, planId)
        validate_str = validate_request(sk, cytxToken, planId)
        if validate_str is not None:
            return validate_str
        
        # 获取方案数据
        response_report_data = requests_select_report(cytxToken, planId)
        logger.info("response_report_data: %s", response_report_data)
        logger.info(type(response_report_data.get("jsonResult")))
        report_data_data = safe_json_loads(response_report_data.get("jsonResult"))
        logger.info(type(report_data_data))
        
        # 获取养老目标，计算养老目标，获取财务报告
        response_target_list = requests_get_target_list(cytxToken, planId, None, None)
        logger.info("response_target_list: %s", response_target_list)
        
        request_calc_target_list_str = {}
        request_calc_target_list_str["mateTargetList"] = response_target_list.get("mateTargetList", [])
        request_calc_target_list_str["selfTargetList"] = response_target_list.get("selfTargetList", [])
        request_calc_target_list_str["planId"] = planId
        
        logger.info("request_calc_target_list_str: %s", request_calc_target_list_str)
        
        response_calc_target_list = requests_calc_target_list(cytxToken, planId, request_calc_target_list_str)
        logger.info("calc_target_list: %s", response_calc_target_list)
        
        request_finIncome = safe_get(
            response_calc_target_list,
            path=["financialDetailsList", 0, "cashFlowList", 1, "amount"],
            default=0.0
        )
        logger.info("request_finIncome: %s", request_finIncome)
        response_plan_report = requests_get_plan_report(cytxToken, planId, request_finIncome)
        logger.info("response_plan_report: %s", response_plan_report)
        
        # 开始解析报告
        report_model = convert_report_to_model(report_data_data, target_list=response_target_list,
                                               calc_target_list=response_calc_target_list,
                                               plan_report=response_plan_report)
        
        logger.info("get_info return: %s", report_model.model_dump_json())
        return report_model.model_dump_json()
    
    except Exception as e:
        # 记录异常信息
        logger.error("Error: %s", e)
        return json.dumps({"error": "Internal server"})


def convert_report_to_model(report_data, *args, **kwargs) -> ReportModel:
    """
    将报告数据转换为模型
    """
    # 先匹配一遍，把完全相同的赋值一遍
    report_model = json_to_model(report_data)
    
    target_list = kwargs.get("target_list")
    calc_target_list = kwargs.get("calc_target_list")
    plan_report = kwargs.get("plan_report")
    
    # 然后手动处理有逻辑的
    report_model.num = get_report_num(report_data)
    report_model.gapAmount = calculate_gap_amount(report_model.payAmount, report_model.amount)
    
    current_age = target_list.get("currentAge")
    mate_current_age= target_list.get("mateCurrentAge",None)
    
    report_model.financialAssetCashFlowList = add_ages_to_cashflow(calc_target_list.get("financialAssetCashFlowList",[]), current_age,mate_current_age)
    report_model.financialDetailsList = add_age_to_nested_cashflow(calc_target_list.get("financialDetailsList",[]), current_age,mate_current_age)
    report_model.astCfgProp = plan_report.get("solutionData", []).get("astCfgProp", [])
    report_model.selfInsCalcData = plan_report.get("selfInsCalcData", [])
    report_model.mateInsCalcData = plan_report.get("mateInsCalcData", [])
    report_model.diagnosisData = plan_report.get("diagnosisData", [])
    
    # 计算百分比
    report_model.assetsTypeAmountList = calculate_assetstypeamountlist_percentages(report_model.assetsTypeAmountList)
    
    return report_model


@mcp.tool(name="getTemplate", description="template")
def get_template(sk: str) -> str:
    logger.info("into get_template")
    # 假设 cytx_report_mcp_server.py 所在目录下有一个 template 文件夹
    TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "template")
    
    # 校验 sk（根据你的需求保留）
    if sk != mcp_secret_key:
        logger.error("Invalid Secret Key")
        return "Invalid Secret Key"
    
    template_path = os.path.join(TEMPLATE_DIR, "template01.json")
    
    if not os.path.exists(template_path):
        logger.error("Template file not found: %s", template_path)
        return json.dumps({"error": "Template not found"})
    
    try:
        with open(template_path, "r", encoding="utf-8") as f:
            template_content = f.read()
        return template_content
    except Exception as e:
        logger.error("Error reading template file: %s", e)
        return json.dumps({"error": "Failed to read template"})

    


def get_report_num(report_data) -> int:
    """
    获取报告数字
    """
    curr_age = report_data.get("currAge")
    
    if curr_age >= 65:
        # 摔老期
        return 4
    elif curr_age >= 50:
        # 成熟期
        return 3
    elif curr_age >= 35:
        # 成长期
        return 2
    elif curr_age >= 25:
        # 形成期
        return 1
    elif curr_age >= 18:
        # 单身期
        return 0


def calculate_gap_amount(pay_amount: Optional[float], amount: Optional[float]) -> float:
    """
    计算金额差距
    """
    return (pay_amount or 0.0) - (amount or 0.0)


def calculate_assetstypeamountlist_percentages(items: list) -> list:
    """
    遍历列表，并在每个元素中添加 percent 字段，表示该元素 amount 占总量的百分比（字符串形式 + 百分号）。

    参数:
        items (list): 包含 "amount" 字段的字典列表

    返回:
        list: 修改后的列表，每个元素中包含新的 percent 字段（str 类型，带 %）
    """
    try:
        total = sum(float(item.amount) for item in items)
    except Exception as e:
        logger.warning("无法解析 amount 数据: %s", e)
        return items  # 返回原始数据，不添加 percent 字段

    if total == 0:
        return items  # 总金额为 0 时不计算百分比

    for item in items:
        amount = float(item.amount)
        percentage = round((amount / total) * 100, 2)
        item.percent = f"{percentage:.2f}%"  # 添加带百分号的字符串

    return items


# 写个方法，校验请求参数
def validate_request(sk: str, cytxToken: str, planId: str) -> str:
    if sk != mcp_secret_key:
        logger.error("Invalid Secret Key")
        return "Invalid Secret Key"
    
    if cytxToken is None:
        logger.error("cytxToken is None")
        return "cytxToken is None"
    
    if planId is None:
        logger.error("planId is None")
        return "planId is None"
    
    return None

def add_ages_to_cashflow(cash_flow_list, current_age, mate_current_age):
    """
    遍历现金流水列表，并根据初始年龄，为每一项添加 age 和（可选）mate_age 字段。

    :param cash_flow_list: 包含 year 字段的字典列表
    :param current_age: 用户当前年龄
    :param mate_current_age: 配偶当前年龄（可能为 None）
    :return: 增加了 age 和（可选）mate_age 字段的新列表
    """
    if not cash_flow_list or not isinstance(cash_flow_list, list):
        return []
    
    base_year = datetime.now().year  # 使用当前年份作为基准年
    result = []

    for item in cash_flow_list:
        year = int(item["year"])
        years_diff = year - base_year
        user_age = current_age + years_diff

        # 创建新字典并更新用户年龄
        new_item = item.copy()
        new_item["age"] = user_age

        # 判断是否添加配偶年龄
        if mate_current_age is not None:
            mate_age = mate_current_age + years_diff
            new_item["spouseAge"] = mate_age

        result.append(new_item)

    return result

from copy import deepcopy
from datetime import datetime


def add_age_to_nested_cashflow(nested_cashflow_list, current_age, mate_current_age=None):
    """
    遍历嵌套的现金流水列表，并根据初始年龄，为每个 cashFlowList 中的项添加 age 和（可选）spouseAge 字段。

    :param nested_cashflow_list: 包含多个 cashFlowList 的字典列表
    :param current_age: 用户当前年龄
    :param mate_current_age: 配偶当前年龄（可能为 None）
    :return: 增加了 age 和（可选）spouseAge 字段的新列表
    """
    if not nested_cashflow_list or not isinstance(nested_cashflow_list, list):
        return []

    result = []
    for item in nested_cashflow_list:
        # 深拷贝确保不影响原始数据
        item_copy = deepcopy(item)
        cash_flow_list = item_copy.get("cashFlowList", [])

        # 使用已有函数处理每个 cashFlowList
        updated_cash_flow = add_ages_to_cashflow(cash_flow_list, current_age, mate_current_age)

        # 替换为更新后的 cashFlowList
        item_copy["cashFlowList"] = updated_cash_flow
        result.append(item_copy)

    return result

def main():
    mcp.run(transport="sse" port=8080)

if __name__ == "__main__":
    # Initialize and run the server
    main()



import json
import logging
import re
import time
from datetime import datetime
from logging.handlers import TimedRotatingFileHandler
from typing import Dict, Any, Union, List, Optional

from mcp.server.fastmcp import FastMCP

from config.settings import mcp_secret_key, mcp_server_host, mcp_server_port, type_code_list, mcp_name, mcp_log_name
from models.command_model import CommandModel
from services.common_utils import is_empty
from services.mcp_service import MCPService
from services.requests_service import requests_get_solution, requests_get_insurance, requests_get_plan_data, \
    requests_get_target_list

# 初始化日志
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] %(name)s: %(message)s',
    handlers=[
        TimedRotatingFileHandler(mcp_log_name, when="D", backupCount=7),
        logging.StreamHandler()
    ]
)
logger = logging.getLogger(__name__)
logging.getLogger("mcp.server.sse").setLevel(logging.DEBUG)


dependencies=[
    "websockets"
]

mcp = FastMCP(mcp_name,port=int(mcp_server_port),host=mcp_server_host,dependencies=dependencies)


# @mcp.tool(name="getPlanData",description="获取养老规划基本信息")
def getPlanData(sk:str,cytxToken:str,planId:str) -> str:
    """
    获取养老规划的基本信息。
    
    参数:
    - sk: 秘钥，用于验证请求合法性。
    - cytxToken: 用户令牌，用于身份验证。
    - planId: 养老规划的唯一标识符。
    
    返回:
    - JSON格式的字符串，包含养老规划的基本信息。
    """
    logger.info("into getPlanData")
    # 记录方法执行时间
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s", sk, cytxToken, planId)
    
    # 校验请求参数是否合法
    validate_str = validate_request(sk,cytxToken, planId)
    if validate_str is not None:
        return validate_str
    
    try:
        # 调用服务层方法获取养老规划基本信息
        retData = requests_get_plan_data(cytxToken,planId)
    except Exception as e:
        # 记录异常信息
        logger.error("Error: %s", e)
        return json.dumps({"error": "requests_get_plan_data error"})
    
    # 记录方法执行时间
    end_time = time.time()
    execution_time = end_time - start_time
    logger.info("Execution time: %.2f seconds", execution_time)
    return json.dumps(retData)


# @mcp.tool(name="getAgedInsurance",description="获取养老规划保险保障信息")
def getAgedInsurance(sk:str,cytxToken:str,planId:str) -> str:
    logger.info("into getAgedInsurance")
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s", sk, cytxToken, planId)
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    try:
        retData = requests_get_insurance(cytxToken, planId)
    except Exception as e:
        logger.error("Error: %s", e)
        return json.dumps({"error": "requests_get_insurance error"})
    # 如何打印方法执行时间
    end_time = time.time()
    execution_time = end_time - start_time
    logger.info("Execution time: %.2f seconds", execution_time)
    return json.dumps(retData)

# @mcp.tool(name="getTargetList",description="获取养老规划目标信息")
def getTargetList(sk:str,cytxToken:str,planId:str) -> str:
    logger.info("into getTargetList")
    # 如何打印方法执行时间
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s", sk, cytxToken, planId)
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    try:
        retData = requests_get_target_list(cytxToken, planId)
    except Exception as e:
        logger.error("Error: %s", e)
        return json.dumps({"error": "requests_get_target_list error"})
    
    # 如何打印方法执行时间
    end_time = time.time()
    execution_time = end_time - start_time
    logger.info("Execution time: %.2f seconds", execution_time)
    return json.dumps(retData)

# @mcp.tool(name="getSolution",description="获取养老规划解决方案")
def getSolution(sk:str,cytxToken:str,planId:str) -> str:
    logger.info("into getSolution")
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s", sk, cytxToken, planId)
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    try:
        retData = requests_get_solution(cytxToken, planId)
    except Exception as e:
        logger.error("Error: %s", e)
        return json.dumps({"error": "requests_get_solution error"})
    
    # 如何打印方法执行时间
    end_time = time.time()
    execution_time = end_time - start_time
    logger.info("Execution time: %.2f seconds", execution_time)
    return json.dumps(retData)

@mcp.tool(name="updateBasicInfo", description="更新")
def updateBasicInfo(sk: str, cytxToken: str, planId: str, reqStr: Union[Dict[str, Any], List[Dict[str, Any]], str, None]) -> str:
    """
    更新养老规划的相关信息。
    
    参数:
    - sk: 秘钥，用于验证请求合法性。
    - cytxToken: 用户令牌，用于身份验证。
    - planId: 养老规划的唯一标识符。
    - reqStr: 请求数据，可以是JSON字符串或字典列表。
    
    返回:
    - JSON格式的字符串，包含更新操作的结果。
    """
    logger.info("into updateBasicInfo")
    # 记录方法执行时间
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s, reqStr=%s", sk, cytxToken, planId, reqStr)
    
    # 校验请求参数是否合法
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    
    try:
        # 将请求数据转换为JSON格式
        if isinstance(reqStr, str):
            reqJson = json.loads(reqStr)
        else:
            reqJson = reqStr
    except json.JSONDecodeError as e:
        # 记录无效JSON格式的错误
        logger.error("Invalid JSON input: %s", e)
        return "Invalid JSON format"

    # 打印请求参数
    logger.info("Request JSON:%s", json.dumps(reqJson))

    # 将 reqJson 转换为 CommandModel 对象
    try:
        command_models = [CommandModel(**item) for item in reqJson]
    except Exception as e:
        # 记录CommandModel解析失败的错误
        logger.error("Failed to parse CommandModel: %s", e)
        return "Invalid CommandModel format"

    # 初始化结果列表
    results = []
    # 设置变量，用来存储各个分类的数组
    base_lists =[]
    targets_lists = []
    insurance_lists = []
    solutions_lists = []
    
    # 遍历 CommandModel 数组，根据 command 类型分类处理
    for command_model in command_models:
        command_type = command_model.type
        #  根据command和command_type进行分组。 分组逻辑如下，command分为add、update、delete，command_type分为以0，1，2，3，4，5开头的
        try:
            logger.info("command_type: %s", command_type)
            if command_type.startswith("0") or command_type.startswith("1") or command_type.startswith("2") :
                if command_type == "0" or command_type =="0-0-0":
                    # 基本信息都按照更新逻辑走
                    command_model.type = "0-0-0"
                    command_model.command='update'
                base_lists.append(command_model)
            elif command_type.startswith("3") :
                targets_lists.append(command_model)
            elif command_type.startswith("4") :
                insurance_lists.append(command_model)
            elif command_type.startswith("5") :
                solutions_lists.append(command_model)
            else:
                results.append("Unsupported command_type")
        except Exception as e:
            # 捕获异常并记录错误信息
            logger.error(f"Error processing command '{command}': {e}")
            results.append(f"Error processing command '{command}': {str(e)}")
            
    # 处理基本信息
    if base_lists is not None and len(base_lists) > 0:
        try:
            retData = MCPService.update_base_list(base_lists, cytxToken, planId)
            results.append(retData)
        except Exception as e:
            logger.error("Error: %s", e)
            return json.dumps({"error": "update_base_list error"})
    
    # 处理目标信息
    if targets_lists is not None and len(targets_lists) > 0:
        try:
            retData = MCPService.update_target_list(targets_lists, cytxToken, planId)
            results.append(retData)
        except Exception as e:
            logger.error("Error: %s", e)
            return json.dumps({"error": "update_target_list error"})
    
    # 处理保险信息
    if insurance_lists is not None and len(insurance_lists) > 0:
        try:
            retData = MCPService.update_insurance_list(insurance_lists, cytxToken, planId)
            results.append(retData)
        except Exception as e:
            logger.error("Error: %s", e)
            return json.dumps({"error": "update_insurance_list error"})
    
    # 处理解决方案信息
    if solutions_lists is not None and len(solutions_lists) > 0:
        try:
            retData = MCPService.update_solutions_list(solutions_lists, cytxToken, planId)
            results.append(retData)
        except Exception as e:
            logger.error("Error: %s", e)
            return json.dumps({"error": "update_solutions_list error"})
    
    # 记录方法执行时间
    end_time = time.time()
    logger.info("Method execution time: %.2f seconds", end_time - start_time)
    logger.info("Update Results: %s", json.dumps(results))
    # 将所有结果整合成一个数组返回
    return json.dumps(results)

# @mcp.tool(name="mergeBaseInfo",description="合并养老规划基本信息")
def mergeBaseInfo(sk: str, cytxToken: str, planId: str, reqStr: Any) -> Any:
    logger.info("into mergeBaseInfo")
    #  如何打印方法执行时间
    start_time = time.time()
    
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s, reqStr=%s", sk, cytxToken, planId, reqStr)
    
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    
    try:
        if isinstance(reqStr, str):
            reqJson = json.loads(reqStr)
        else:
            reqJson = reqStr
    except json.JSONDecodeError as e:
        logger.error("Invalid JSON input: %s", e)
        return "Invalid JSON format"
        
    # 打印请求参数reqJson
    logger.info("Request JSON:%s", json.dumps(reqJson))
    
    try:
        MCPService.handle_merge_logic(reqJson, cytxToken, planId)
    except Exception as e:
        logger.error("merge fail: %s", e)
        return json.dumps({"error": "merge fail"})
    # 如何打印方法执行时间
    end_time = time.time()
    logger.info("Method execution time: %.2f seconds", end_time - start_time)
    return json.dumps({"status": "OK"})

@mcp.tool(name="queryInfo", description="查询")
def queryInfo(sk: str, cytxToken: str, planId: str, reqStr: Union[Dict[str, Any], List[Dict[str, Any]], str, None]) -> str:
    """
    查询养老规划的所有相关信息。
    
    参数:
    - sk: 秘钥，用于验证请求合法性。
    - cytxToken: 用户令牌，用于身份验证。
    - planId: 养老规划的唯一标识符。
    - reqStr: 请求数据，包含type和ownertype字段。
    
    返回:
    - JSON格式的字符串，包含所有查询到的信息。
    """
    logger.info("into queryInfo")
    # 记录方法执行时间
    start_time = time.time()
    # 打印接收到的参数
    logger.info("Received parameters: sk=%s, cytxToken=%s, planId=%s, reqStr=%s", sk, cytxToken, planId, reqStr)
    
    # 校验请求参数是否合法
    validate_str = validate_request(sk, cytxToken, planId)
    if validate_str is not None:
        return validate_str
    
    try:
        # 将请求数据转换为JSON格式
        if isinstance(reqStr, str):
            reqJson = json.loads(reqStr)
        else:
            reqJson = reqStr
    except json.JSONDecodeError as e:
        # 记录无效JSON格式的错误
        logger.error("Invalid JSON input: %s", e)
        return "Invalid JSON format"

    # 打印请求参数
    logger.info("Request JSON:%s", json.dumps(reqJson))
    
    # 判断下，如果reqJson是数组，只取第一个元素
    reqJson = reqJson[0] if isinstance(reqJson, list) else reqJson

    # 初始化结果字典
    results = {}

    # 根据type和ownertype字段调用不同的查询方法
    try:
        search_age = None
        if "searchAge" in reqJson:
            search_age = reqJson["searchAge"]
            
        
        if "type" in reqJson and "ownerType" in reqJson:
            type_value = reqJson["type"]
            ownertype_value = reqJson["ownerType"]
            
            if is_empty(ownertype_value):
                ownertype_value = "0"
                
            if is_empty(type_value):
                type_value = "0-0-0"
            
            # 进行格式校验
            if re.match(r'^\d+(-\d+)*$', type_value):
                pass
            elif re.match(r'^[\u4e00-\u9fa5]+(-[\u4e00-\u9fa5]+)*(-[\u4e00-\u9fa5]+)*$', type_value):
                type_value = get_type_code_by_name(type_value)
            else:
                return json.dumps({"error": "Invalid type format"})
            
            # 查询基础信息
            if type_value.startswith("0") :
                base_info = requests_get_plan_data(cytxToken, planId)
                find_result = MCPService.find_data_by_type_new(type_value, base_info, ownertype_value, None, None)
                
            elif type_value.startswith("1") :
                base_info = requests_get_plan_data(cytxToken, planId)
                find_result = MCPService.find_data_by_type_new(type_value, base_info, ownertype_value, None, None)
                
            elif type_value.startswith("2") :
                if search_age is not None and type_value == "2-1-1" :
                    base_info = requests_get_plan_data(cytxToken, planId,need_filter=False)
                    search_age = calculate_search_age(base_info,search_age, ownertype_value)
                    find_result = MCPService.find_data_by_type_new(type_value, base_info, ownertype_value, None, None)
                    find_result = filter_by_age(find_result, search_age)
                else:
                    base_info = requests_get_plan_data(cytxToken, planId)
                    find_result = MCPService.find_data_by_type_new(type_value, base_info, ownertype_value, None, None)

            # 查询目标信息
            elif type_value.startswith("3") :
                target_info = requests_get_target_list(cytxToken, planId,None, None)
                find_result = MCPService.find_data_by_type_new(type_value, target_info, ownertype_value, None, None)

            # 查询保险信息
            elif type_value.startswith("4"):
                insurance_info = requests_get_insurance(cytxToken, planId)
                find_result = MCPService.find_data_by_type_new(type_value, insurance_info, ownertype_value, None, None)

            # 查询解决方案信息
            elif type_value.startswith("5"):
                solution_info = requests_get_solution(cytxToken, planId)
                find_result = MCPService.find_data_by_type_new(type_value, solution_info, ownertype_value, None, None)
            # 如果ownertype为特定值，可以扩展其他逻辑
            else:
                pass
            
            if find_result:
                # 房产需要单独处理，因为房产id居然都是一致的
                result_items = [item[0] for item in find_result if item[0] is not None]
                if type_value == "1-5-14" :
                    result_items = filter_house_id(result_items)
            else:
                result_items = []
            
            results["data"] = result_items
        else:
            logger.error("Invalid request parameters")
            return json.dumps({"error": "Invalid request parameters"})

    except Exception as e:
        # 捕获异常并记录错误信息
        logger.error("Error querying information: %s", e)
        return json.dumps({"error": "Error querying information"})

    # 记录方法执行时间
    end_time = time.time()
    logger.info("Method execution time: %.2f seconds", end_time - start_time)
    logger.info("Query results: %s", json.dumps(results))
    # 返回所有查询结果
    return json.dumps(results)

# 写个方法，校验请求参数
def validate_request(sk: str, cytxToken: str, planId: str) -> str:
    if sk != mcp_secret_key:
        logger.error("Invalid Secret Key")
        return "Invalid Secret Key"
    
    if cytxToken is None:
        logger.error("cytxToken is None")
        return "cytxToken is None"
    
    if planId is None:
        logger.error("planId is None")
        return "planId is None"
    
    return None

def filter_house_id(house_list):
    """
    过滤掉房产id
    """
    for house in house_list:
        if house["id"]:
            house.pop('id')
    return house_list

def get_type_code_by_name(type_name: str) -> str:
    for type_code in type_code_list:
        if type_code["name"] == type_name:
            return type_code["code"]
    return ""

def filter_by_age(data_list: List[Dict[str, Any]], search_age: int) -> List[Dict[str, Any]]:
    """
    根据 search_age 过滤数据列表。

    Args:
        data_list (List[Dict[str, Any]]): 数据列表。
        search_age (int): 搜索年龄。

    Returns:
        List[Dict[str, Any]]: 过滤后的数据列表。
    """
    logger.info("into filter_by_age")
    logger.info("data_list: %s", data_list)
    logger.info("search_age: %s", search_age)
    for item in data_list:
        if item[0]["desp"] ==  "工作收入":
            am = get_optimal_amt_by_age(item[0].get("trendDTO", {}),  search_age)
            if am is not None:
                item[0]["amount"] = am
    
    # 过滤trendDTO
    for item in data_list:
        item[0].pop("trendDTO", None)
    
    return data_list
    
    
    
def calculate_search_age(data: Dict[str, Any],search_age: Optional[int],ownertype_value: str) -> int:
    """
   计算 search_age 的值。

   如果 search_age < 100，直接返回；
   如果 search_age >= 100，认为是年份，如2025年，计算当前年龄。

   Args:
       search_age (Optional[int]): 输入的 search_age 值。

   Returns:
       int: 计算后的年龄。
   """
    logger.info("into calculate_search_age")
    if search_age is None:
        return None
    if search_age < 100:
        return search_age
    if search_age >= 2000:
        if str(ownertype_value) == "0":
            current_age = data.get("planInfo").get("currentAge")
        else:
            current_age = data.get("planInfo").get("mateCurrentAge")
        
        # 根据当前年龄和当前年份，计算search_age时的年龄
        # 获取当前年份
        current_year = datetime.now().year
        return current_age + (search_age - current_year)
        
    else:
        return None


def get_optimal_amt_by_age(data: dict, age: int) -> Optional[float]:
    """
    根据传入的年龄，在 data['ageArr'] 中查找对应的索引，
    并返回 data['optimalAmtList'][index] 的值。

    Args:
        data (dict): 包含 ageArr 和 optimalAmtList 的数据字典。
        age (int): 要查询的年龄。

    Returns:
        Optional[float]: 查询结果，如果找不到则返回 None。
    """
    age_list = data.get("ageArr", [])
    optimal_amt_list = data.get("optimalAmtList", [])
    
    try:
        index = age_list.index(age)
        return optimal_amt_list[index]
    except ValueError:
        logger.warning(f"Age {age} not found in ageArr.")
        return None
    except IndexError:
        logger.error(f"Index error when accessing optimalAmtList for age {age}.")
        return None

def main():
    mcp.run(transport="sse")

if __name__ == "__main__":
    # Initialize and run the server
    main()
